@@ -37,12 +37,10 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
 import java.util.Collections;
-import java.util.Date;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.LinkedHashMap;
 import java.util.List;
-import java.util.Locale;
 import java.util.Map;
 import java.util.ResourceBundle;
 import java.util.Set;
@@ -71,16 +69,17 @@ import org.apache.cxf.helpers.CastUtils;
 import org.apache.cxf.jaxrs.ext.MessageContext;
 import org.apache.cxf.jaxrs.ext.ParameterHandler;
 import org.apache.cxf.jaxrs.ext.ProtocolHeaders;
+import org.apache.cxf.jaxrs.ext.search.SearchContext;
 import org.apache.cxf.jaxrs.impl.MetadataMap;
 import org.apache.cxf.jaxrs.impl.PathSegmentImpl;
 import org.apache.cxf.jaxrs.impl.tl.ThreadLocalContextResolver;
 import org.apache.cxf.jaxrs.impl.tl.ThreadLocalHttpHeaders;
-import org.apache.cxf.jaxrs.impl.tl.ThreadLocalInvocationHandler;
 import org.apache.cxf.jaxrs.impl.tl.ThreadLocalMessageContext;
 import org.apache.cxf.jaxrs.impl.tl.ThreadLocalProtocolHeaders;
 import org.apache.cxf.jaxrs.impl.tl.ThreadLocalProviders;
 import org.apache.cxf.jaxrs.impl.tl.ThreadLocalProxy;
 import org.apache.cxf.jaxrs.impl.tl.ThreadLocalRequest;
+import org.apache.cxf.jaxrs.impl.tl.ThreadLocalSearchContext;
 import org.apache.cxf.jaxrs.impl.tl.ThreadLocalSecurityContext;
 import org.apache.cxf.jaxrs.impl.tl.ThreadLocalUriInfo;
 import org.apache.cxf.jaxrs.model.AbstractResourceInfo;
@@ -88,7 +87,6 @@ import org.apache.cxf.jaxrs.model.Parameter;
 import org.apache.cxf.jaxrs.model.ParameterType;
 import org.apache.cxf.jaxrs.provider.ProviderFactory;
 import org.apache.cxf.message.Message;
-import org.apache.cxf.message.MessageUtils;
 
 public final class InjectionUtils {
     
@@ -100,8 +98,6 @@ public final class InjectionUtils {
     private static final String HTTP_SERVLET_REQUEST_CLASS_NAME = "javax.servlet.http.HttpServletRequest";
     private static final String HTTP_SERVLET_RESPONSE_CLASS_NAME = "javax.servlet.http.HttpServletResponse";
         
-    private static final String PARAM_HANDLERS_FIRST = "check.parameter.handlers.first";
-    
     private InjectionUtils() {
         
     }
@@ -110,10 +106,10 @@ public final class InjectionUtils {
         return !cls.isInterface() && !Modifier.isAbstract(cls.getModifiers());
     }
     
-    public static Type getSuperType(Class<?> serviceClass, TypeVariable<?> var) {
+    public static Type getSuperType(Class<?> serviceClass, TypeVariable var) {
         
         int pos = 0;
-        TypeVariable<?>[] vars = var.getGenericDeclaration().getTypeParameters();
+        TypeVariable[] vars = var.getGenericDeclaration().getTypeParameters();
         for (; pos < vars.length; pos++) {
             if (vars[pos].getName().equals(var.getName())) {
                 break;
@@ -156,10 +152,11 @@ public final class InjectionUtils {
         return methodToInvoke; 
     }
  
+    @SuppressWarnings("unchecked")
     public static void injectFieldValue(final Field f, 
                                         final Object o, 
                                         final Object v) {
-        AccessController.doPrivileged(new PrivilegedAction<Object>() {
+        AccessController.doPrivileged(new PrivilegedAction() {
             public Object run() {
                 f.setAccessible(true);
                 try {
@@ -174,9 +171,10 @@ public final class InjectionUtils {
         
     }
 
+    @SuppressWarnings("unchecked")
     public static Object extractFieldValue(final Field f, 
                                         final Object o) {
-        return AccessController.doPrivileged(new PrivilegedAction<Object>() {
+        return AccessController.doPrivileged(new PrivilegedAction() {
             public Object run() {
                 f.setAccessible(true);
                 try {
@@ -202,7 +200,7 @@ public final class InjectionUtils {
         }
         if (!ParameterizedType.class.isAssignableFrom(genericType.getClass())) {
             if (genericType instanceof TypeVariable) {
-                genericType = getType(((TypeVariable<?>)genericType).getBounds(), pos);
+                genericType = getType(((TypeVariable)genericType).getBounds(), pos);
             } else if (genericType instanceof WildcardType) { 
                 WildcardType wildcardType = (WildcardType)genericType;
                 Type[] bounds = wildcardType.getLowerBounds();
@@ -232,12 +230,12 @@ public final class InjectionUtils {
         if (genericType == null) {
             return null;
         } else if (genericType instanceof Class) {
-            return (Class<?>) genericType;
+            return (Class) genericType;
         } else if (genericType instanceof ParameterizedType) {
             ParameterizedType paramType = (ParameterizedType)genericType;
             Type t = paramType.getRawType();
             if (t instanceof Class) {
-                return (Class<?>)t;
+                return (Class)t;
             }
         }
         // it might be a TypeVariable, or a GenericArray.
@@ -280,19 +278,20 @@ public final class InjectionUtils {
         return null;
     }
     
-    public static <T> T handleParameter(String value, 
-                                        boolean decoded,
-                                        Class<T> pClass,
-                                        Annotation[] paramAnns,
-                                        ParameterType pType,
-                                        Message message) {
+    public static Object handleParameter(String value, 
+                                         boolean decoded,
+                                         Class<?> pClass,
+                                         Annotation[] paramAnns,
+                                         ParameterType pType,
+                                         Message message) {
+        
         if (value == null) {
             return null;
         }
         
         if (pType == ParameterType.PATH) {
             if (PathSegment.class.isAssignableFrom(pClass)) {
-                return pClass.cast(new PathSegmentImpl(value, decoded));   
+                return new PathSegmentImpl(value, decoded);   
             } else {
                 value = new PathSegmentImpl(value, false).getPath();                 
             }
@@ -300,14 +299,10 @@ public final class InjectionUtils {
         
         value = decodeValue(value, decoded, pType);
         
+        
         if (pClass.isPrimitive()) {
             try {
-                @SuppressWarnings("unchecked")
-                T ret = (T)PrimitiveUtils.read(value, pClass);
-                // cannot us pClass.cast as the pClass is something like
-                // Boolean.TYPE (representing the boolean primitive) and
-                // the object is a Boolean object
-                return ret;
+                return PrimitiveUtils.read(value, pClass);
             } catch (NumberFormatException nfe) {
                 //
                 //  For path, query & matrix parameters this is 404,
@@ -322,27 +317,23 @@ public final class InjectionUtils {
         }
         
         boolean adapterHasToBeUsed = false;
-        Class<?> cls = pClass;        
         Class<?> valueType = JAXBUtils.getValueTypeFromAdapter(pClass, pClass, paramAnns);
-        if (valueType != cls) {
-            cls = valueType;
+        if (valueType != pClass) {
+            pClass = valueType;
             adapterHasToBeUsed = true;
         }
         
-        Object result = instantiateFromParameterHandler(value, cls, message);
-        if (result != null) {
-            return pClass.cast(result);
-        }
+        Object result = null;
         // check constructors accepting a single String value
         try {
-            Constructor<?> c = cls.getConstructor(new Class<?>[]{String.class});
+            Constructor<?> c = pClass.getConstructor(new Class<?>[]{String.class});
             result = c.newInstance(new Object[]{value});
         } catch (NoSuchMethodException ex) {
             // try valueOf
         } catch (WebApplicationException ex) {
             throw ex;
         } catch (Exception ex) {
-            result = createFromParameterHandler(value, cls, message);
+            result = createFromParameterHandler(value, pClass, message);
             if (result == null) {
                 LOG.severe(new org.apache.cxf.common.i18n.Message("CLASS_CONSTRUCTOR_FAILURE", 
                                                                    BUNDLE, 
@@ -352,11 +343,11 @@ public final class InjectionUtils {
         }
         if (result == null) {
             // check for valueOf(String) static methods
-            String[] methodNames = cls.isEnum() 
+            String[] methodNames = pClass.isEnum() 
                 ? new String[] {"fromString", "fromValue", "valueOf"} 
                 : new String[] {"valueOf", "fromString"};
             for (String mName : methodNames) {   
-                result = evaluateFactoryMethod(value, cls, pType, mName);
+                result = evaluateFactoryMethod(value, pClass, pType, mName);
                 if (result != null) {
                     break;
                 }
@@ -364,7 +355,7 @@ public final class InjectionUtils {
         }
         
         if (result == null) {
-            result = createFromParameterHandler(value, cls, message);
+            result = createFromParameterHandler(value, pClass, message);
         }
         
         if (result != null && adapterHasToBeUsed) {
@@ -380,26 +371,15 @@ public final class InjectionUtils {
             reportServerError("WRONG_PARAMETER_TYPE", pClass.getName());
         }
         
-        return pClass.cast(result);
+        return result;
     }
 
-    private static <T> T instantiateFromParameterHandler(String value, 
-                                                     Class<T> pClass,
-                                                     Message m) {
-        if (Date.class == pClass || Locale.class == pClass 
-            || m != null && MessageUtils.isTrue(m.getContextualProperty(PARAM_HANDLERS_FIRST))) {
-            return createFromParameterHandler(value, pClass, m);
-        } else {
-            return null;
-        }
-    }
-    
-    private static <T> T createFromParameterHandler(String value, 
-                                                    Class<T> pClass,
-                                                    Message message) {
-        T result = null;
+    private static Object createFromParameterHandler(String value, 
+                                                     Class<?> pClass,
+                                                     Message message) {
+        Object result = null;
         if (message != null) {
-            ParameterHandler<T> pm = ProviderFactory.getInstance(message)
+            ParameterHandler<?> pm = ProviderFactory.getInstance(message)
                 .createParameterHandler(pClass);
             if (pm != null) {
                 result = pm.fromString(value);
@@ -420,14 +400,14 @@ public final class InjectionUtils {
         throw new WebApplicationException(r);
     }
     
-    private static <T> T evaluateFactoryMethod(String value,
-                                                Class<T> pClass, 
+    private static Object evaluateFactoryMethod(String value,
+                                                Class<?> pClass, 
                                                 ParameterType pType, 
                                                 String methodName) {
         try {
             Method m = pClass.getMethod(methodName, new Class<?>[]{String.class});
             if (Modifier.isStatic(m.getModifiers())) {
-                return pClass.cast(m.invoke(null, new Object[]{value}));
+                return m.invoke(null, new Object[]{value});
             }
         } catch (NoSuchMethodException ex) {
             // no luck
@@ -573,7 +553,7 @@ public final class InjectionUtils {
         if (first == null) {
             return second;
         } else if (first instanceof Map) {
-            Map.class.cast(first).putAll((Map<?, ?>) second);
+            Map.class.cast(first).putAll((Map) second);
             return first;
         }
         return null;
@@ -699,7 +679,7 @@ public final class InjectionUtils {
         if (first == null) {
             return second;
         } else if (first instanceof Collection) {
-            Collection.class.cast(first).addAll((Collection<?>) second);
+            Collection.class.cast(first).addAll((Collection) second);
             return first;
         } else {
             int firstLen = Array.getLength(first);
@@ -844,9 +824,8 @@ public final class InjectionUtils {
     }
     
     // TODO : investigate the possibility of using generic proxies only
-    @SuppressWarnings("unchecked")
-    public static <T> ThreadLocalProxy<T> createThreadLocalProxy(Class<T> type) {
-        ThreadLocalProxy<?> proxy = null;
+    public static ThreadLocalProxy createThreadLocalProxy(Class<?> type) {
+        ThreadLocalProxy proxy = null;
         if (UriInfo.class.isAssignableFrom(type)) {
             proxy = new ThreadLocalUriInfo();
         } else if (HttpHeaders.class.isAssignableFrom(type)) {
@@ -856,11 +835,13 @@ public final class InjectionUtils {
         } else if (SecurityContext.class.isAssignableFrom(type)) {
             proxy = new ThreadLocalSecurityContext();
         } else if (ContextResolver.class.isAssignableFrom(type)) {
-            proxy = new ThreadLocalContextResolver<Object>();
+            proxy = new ThreadLocalContextResolver();
         } else if (Request.class.isAssignableFrom(type)) {
             proxy = new ThreadLocalRequest();
         }  else if (Providers.class.isAssignableFrom(type)) {
             proxy = new ThreadLocalProviders();
+        } else if (SearchContext.class.isAssignableFrom(type)) {
+            proxy = new ThreadLocalSearchContext();
         } else if (MessageContext.class.isAssignableFrom(type)) {
             proxy = new ThreadLocalMessageContext();
         }
@@ -868,20 +849,15 @@ public final class InjectionUtils {
         if (proxy == null && isServletApiContext(type.getName())) {
             proxy = createThreadLocalServletApiContext(type.getName());  
         }
-        if (proxy == null) {
-            return (ThreadLocalProxy<T>)Proxy.newProxyInstance(type.getClassLoader(),
-                                   new Class[] {type, ThreadLocalProxy.class },
-                                   new ThreadLocalInvocationHandler<T>());
-        }
         
-        return (ThreadLocalProxy<T>)proxy;
+        return proxy;
     }
     
     private static boolean isServletApiContext(String name) { 
         return name.startsWith("javax.servlet.");
     }
     
-    private static ThreadLocalProxy<?> createThreadLocalServletApiContext(String name) {
+    private static ThreadLocalProxy createThreadLocalServletApiContext(String name) {
         String proxyClassName = null;
         if (HTTP_SERVLET_REQUEST_CLASS_NAME.equals(name)) {
             proxyClassName = "org.apache.cxf.jaxrs.impl.tl.ThreadLocalHttpServletRequest";
@@ -893,7 +869,7 @@ public final class InjectionUtils {
             proxyClassName = "org.apache.cxf.jaxrs.impl.tl.ThreadLocalServletConfig";
         }
         try {
-            return (ThreadLocalProxy<?>)ClassLoaderUtils.loadClass(proxyClassName, InjectionUtils.class)
+            return (ThreadLocalProxy)ClassLoaderUtils.loadClass(proxyClassName, InjectionUtils.class)
                 .newInstance();
         } catch (Throwable t) {
             throw new RuntimeException(t);
@@ -918,6 +894,11 @@ public final class InjectionUtils {
             Object value = f.getType() == Application.class ? app : cri.getContextFieldProxy(f);
             InjectionUtils.injectFieldValue(f, instance, value);
         }
+        
+        for (Field f : cri.getResourceFields()) {
+            ThreadLocalProxy proxy = cri.getResourceFieldProxy(f);
+            InjectionUtils.injectFieldValue(f, instance, proxy);
+        }
     }
     
     public static void injectContextProxies(AbstractResourceInfo cri, Object instance) {
@@ -926,11 +907,13 @@ public final class InjectionUtils {
     
     @SuppressWarnings("unchecked")
     public static void injectContextField(AbstractResourceInfo cri, 
-                                          Field f, Object o, Object value) {
+                                          Field f, Object o, Object value,
+                                          boolean resource) {
         if (!cri.isSingleton()) {
             InjectionUtils.injectFieldValue(f, o, value);
         } else {
-            ThreadLocalProxy<Object> proxy = (ThreadLocalProxy<Object>)cri.getContextFieldProxy(f);
+            ThreadLocalProxy proxy = resource ? cri.getResourceFieldProxy(f)
+                                              : cri.getContextFieldProxy(f);
             if (proxy != null) {
                 proxy.set(value);
             }
@@ -942,6 +925,7 @@ public final class InjectionUtils {
                                  Message message) {
         injectContextMethods(requestObject, resource, message);
         injectContextFields(requestObject, resource, message);
+        injectResourceFields(requestObject, resource, message);
     }
     
     @SuppressWarnings("unchecked")
@@ -962,8 +946,7 @@ public final class InjectionUtils {
                 if (!cri.isSingleton()) {
                     InjectionUtils.injectThroughMethod(requestObject, method, o);
                 } else {
-                    ThreadLocalProxy<Object> proxy 
-                        = (ThreadLocalProxy<Object>)cri.getContextSetterProxy(method);
+                    ThreadLocalProxy proxy = cri.getContextSetterProxy(method);
                     if (proxy != null) {
                         proxy.set(o);
                     }
@@ -984,7 +967,17 @@ public final class InjectionUtils {
                 continue;
             }
             Object value = JAXRSUtils.createContextValue(m, f.getGenericType(), f.getType());
-            InjectionUtils.injectContextField(cri, f, o, value);
+            InjectionUtils.injectContextField(cri, f, o, value, false);
+        }
+    }
+    
+    public static void injectResourceFields(Object o,
+                                            AbstractResourceInfo cri,
+                                            Message m) {
+        
+        for (Field f : cri.getResourceFields()) {
+            Object value = JAXRSUtils.createResourceValue(m, f.getGenericType(), f.getType());
+            InjectionUtils.injectContextField(cri, f, o, value, true);
         }
     }
     
@@ -1102,12 +1095,13 @@ public final class InjectionUtils {
         }
     }
     
-    public static <T> Object convertStringToPrimitive(String value, Class<?> cls) {
+    @SuppressWarnings("unchecked")
+    public static <T> T convertStringToPrimitive(String value, Class<T> cls) {
         if (String.class == cls) {
-            return value;
+            return cls.cast(value);
         }
         if (cls.isPrimitive()) {
-            return PrimitiveUtils.read(value, cls);
+            return (T)PrimitiveUtils.read(value, cls);
         } else {
             try {
                 Method m = cls.getMethod("valueOf", new Class[]{String.class});
